@@ -14,7 +14,7 @@ BEGIN {
 
 use Graph::AdjacencyMap qw(:flags :fields);
 
-our $VERSION = '0.9724';
+our $VERSION = '0.9726';
 
 require 5.006; # Weak references are absolutely required.
 
@@ -1225,7 +1225,16 @@ sub subgraph {
   my $v = Set::Object->new($dst ? grep $g->has_vertex($_), @$dst : @u);
   $s->add_vertices(@u, $dst ? $v->members : ());
   my $directed = &is_directed;
-  $s->add_edges(grep $v->contains($directed ? $_->[1] : @$_), $g->edges_from(@u));
+  if ($directed) {
+    $s->add_edges(grep $v->contains($_->[1]), $g->edges_from(@u));
+  } else {
+    my $valid = $dst ? $v + Set::Object->new(@u) : $v;
+    $s->add_edges(
+      grep +($v->contains($_->[0]) || $v->contains($_->[1])) &&
+        ($valid->contains($_->[0]) && $valid->contains($_->[1])),
+        $g->edges_from(@u)
+    );
+  }
   return $s;
 }
 
